/**
 * Ajax pagination component
 * This component reproduces the server-side pagination  component but using Ajax requests
 * and span.link instead of real links.
 */
loom.ui.AbstractPagedContainer = Class.create({

	/**
	 * url: the URL to request paged data
	 * linksPosition: 'top', 'bottom', 'both', 'none'
	 * linksCount:  # links to display
	 * renderItemCallback: Function to render each data item (must return also the 'li' tag)
	 */
	initialize: function(container, options) {
		this.container = container;
		this.container.addClassName('paged');
		this.options = Object.extend({
			linksPosition: 'top',
			linksCount: 10
		}, options || {});
		this.getPage(0);
		this.container.observe('click', this.onClick.bindAsEventListener(this));
	},
	
	onReceiveData: function(transport) {
		this.renderData(transport.responseText.evalJSON());
	},
	
	renderData: function(data) {
		this.data = data;
		this.container.update();
		if (!data.totalResults) {
			this.container.update(loom.messages.empty);
		} else {
			var lp = this.options.linksPosition;
			(lp === 'top' || lp === 'both') && this.renderLinks(data);
			this.container.insert(this.renderContents(data));
			(lp === 'bottom' || lp === 'both') && this.renderLinks(data);
		}
		this.container.removeClassName('loading');
	},
	
	createLink: function(pageIndex, contents, liClass) {
		var css = pageIndex < 0 || pageIndex >= this.data.totalPages? 'disabled' : pageIndex == this.data.pageIndex? 'active' : '';
		var a = new Element('span', { className: !css && 'link', 'data-page-index': pageIndex }).update(contents);
		return new Element('li', { className: css + (liClass? ' ' + liClass : '') }).update(a);
	},
	
	renderLinks: function(data) {
		if (data.totalPages == 1) {
			return;
		}
		var ul = new Element('ul', { className: 'links' });
		var ellipsis = "<li>&hellip;</li>";
		var linksCount = this.options.linksCount;
		
		// previous and optional ellipsis
		ul.insert(this.createLink(data.pageIndex - 1, loom.messages.previous, 'previous'));
		(data.pageIndex > linksCount / 2)  && ul.insert(ellipsis);
		
		// page links
		var index = Math.max(data.pageIndex - linksCount / 2, 0);
		var max = Math.min(index + linksCount, data.totalPages);
		while (index < max) {
			ul.insert(this.createLink(index, index + 1));
			index++;
		}
		
		// next and optional ellipsis
		(max < data.totalPages) && ul.insert(ellipsis);
		ul.insert(this.createLink(data.pageIndex + 1, loom.messages.next, 'next'));
		
		this.container.insert(ul);
	},
	
	onClick: function(event) {
		var a = event.findElement('.link');
		if (a && !a.match('.disabled .link')) {
			event.stop();
			this.getPage(a.getAttribute('data-page-index'));
		}
	},
	
	getPage: function(pageIndex) {
		this.container.addClassName('loading');
		var d = this.container.down('.data');
		d && d.setStyle('visibility:hidden;');
		new Ajax.Request(this.options.url, {
			method: 'get',
			parameters: { '_paged-index': pageIndex },
			onSuccess: this.onReceiveData.bind(this)
		});
	}

});

loom.ui.PagedList = Class.create(loom.ui.AbstractPagedContainer, {

	/**
	 * Render contents as a UL
	 */
	renderContents: function(data) {
		var contents = new Element('ul', { className: 'data' });
		data.data.each(function(e, index) {
			contents.insert(this.options.renderItemCallback(e, index));
		}.bind(this));
		return contents;
	}

});